#include <iostream>
#include <string>

#include <boost/program_options.hpp>

#include <msgServer.hh>

int main(int argc, char ** argv){
	std::string address;
	std::string port=DEFAULT_SERVER_PORT;
	unsigned int maximumConnections=DEFAULT_MAX_CLIENT_CONNECTIONS;
	bool toggleTime;
	unsigned long int inactivityTime=DEFAULT_SERVER_INACTIVITY_TIME;
	
	//Supported options
	boost::program_options::options_description desc("Allowed options");

	desc.add_options()
		("help", "Produce help message")
		("address,a", boost::program_options::value<std::string>(&address), "The address of the server (default : localhost)")
		("port,p", boost::program_options::value<std::string>(&port), "The port number of the server (default : 5500)")
		("time,t", boost::program_options::value<unsigned long int>(&inactivityTime), "The maximum inactivity time in second before the automatic-shutdown of the server (default : 3600, opposite of noLimit)")
		("noLimit,n", "Deactivate the time limit, the server runs until shutdown by hand (opposite of time)")
		("max-clients,m", boost::program_options::value<unsigned int>(&maximumConnections), "Maximum number of clients that can connect at the same time (default : 10)");
	
	//The order for parameter (without the -.. options) is : address port time
	boost::program_options::positional_options_description positional;
	positional.add("address", 1);
	positional.add("port", 1);
	positional.add("time", 1);

	//Parse the parameters
	boost::program_options::variables_map variables;
	try{
		boost::program_options::store(boost::program_options::command_line_parser(argc, argv).options(desc).positional(positional).run(), variables);
	}catch(boost::program_options::validation_error e){
		//Called if a parameter get a value that does not matchits type
		std::cerr<<"Catch a validation error exception : "<<e.what()<<std::endl;
		exit(99);
	}catch(boost::program_options::unknown_option e){
		//Called if an unknown option is given (e.g. : -z)
		std::cerr<<"Catch an excpetion : "<<e.what()<<std::endl;
	}
	boost::program_options::notify(variables);
	
	//Print usage
	if((variables.count("help")) || (variables.count("time") && variables.count("noLimit"))){
		std::string software(argv[0]);
		std::cout<<"Usage:\tDefault:\t"<<software<<std::endl;
		std::cout<<"\tCompact:\t"<<software<<" address:port [max_time]"<<std::endl;
		std::cout<<"\tStandard:\t"<<software<<" address port [max_time]"<<std::endl;
		std::cout<<"\tComplete:\t"<<software<<" [-a address] [-p port] [-t max_time|-n] [-m max_connections]"<<std::endl<<std::endl;
		std::cout<<desc<<std::endl;
		return 1;
	}
	
	//Allow to write "address:port" (in addition to "address port")
	if(address.find(":")!=std::string::npos){
		//The address contains the port number as well
		size_t pos=address.find(":");
		
		std::string tempPort=address.substr(pos+1);
		if(!tempPort.empty()){
			//currentPort=std::stoi(tempPort, nullptr, 10);
			port=atoi(tempPort.c_str());
		}
		
		if(pos!=0){
			address=address.substr(0, pos);
		}else{
			address="localhost";
		}
	}
	
	toggleTime=!(variables.count("noLimit"));
	
	//Start the server
	msgServer server(address, port, toggleTime, inactivityTime, maximumConnections);
	server.run();
}
